Index: src/Fl_Pixmap.cxx
===================================================================
--- src/Fl_Pixmap.cxx	(revision 10757)
+++ src/Fl_Pixmap.cxx	(working copy)
@@ -40,6 +40,10 @@
 #include <FL/Fl_Pixmap.H>
 #include <FL/Fl_Printer.H>
 
+#if !(defined(__APPLE__) || defined(WIN32))
+#include <X11/Xregion.h>
+#endif
+
 #include <stdio.h>
 #include "flstring.h"
 #include <ctype.h>
@@ -190,30 +194,42 @@
 //------------------------------------------------------------------------------
 #else							// X11, Xlib
 //------------------------------------------------------------------------------
-
 void Fl_Xlib_Graphics_Driver::draw(Fl_Pixmap *pxm, int XP, int YP, int WP, int HP, int cx, int cy) {
   int X, Y, W, H;
+  bool complex_clip = false;
+  Region r = NULL;
   if (pxm->prepare(XP, YP, WP, HP, cx, cy, X, Y, W, H)) return;
   if (pxm->mask_) {
-    // I can't figure out how to combine a mask with existing region,
-    // so cut the image down to a clipped rectangle:
-    int nx, ny; fl_clip_box(X,Y,W,H,nx,ny,W,H);
-    cx += nx-X; X = nx;
-    cy += ny-Y; Y = ny;
     // make X use the bitmap as a mask:
     XSetClipMask(fl_display, fl_gc, pxm->mask_);
-    int ox = X-cx; if (ox < 0) ox += pxm->w();
-    int oy = Y-cy; if (oy < 0) oy += pxm->h();
     XSetClipOrigin(fl_display, fl_gc, X-cx, Y-cy);
+    r = clip_region();
+    if (r && !(XRectInRegion(r, X, Y, W, H) == RectangleIn)) {
+      // At this point, XYWH is the rectangle encompassing the intersection between
+      // the current clip region and the (portion of the) pixmap we have to draw.
+      // The current clip region is often a rectangle. But, when the rounded top angle of a window
+      // is moved above another window, resulting expose events may create a complex clipping region
+      // made of several rectangles. Call copy_offscreen() for each rectangle in the clipping region.
+      complex_clip = true;
+      Region r2 = XRectangleRegion(X,Y,W,H);
+      XIntersectRegion(r2, r, r2); // intersection of clipping region and XYWH
+      for (int i = 0; i < r2->numRects; i++) { // for each rectangle composing r2
+        int X2 = r2->rects[i].x1; int Y2 = r2->rects[i].y1;
+        W = r2->rects[i].x2 - r2->rects[i].x1; H = r2->rects[i].y2 - r2->rects[i].y1;
+        copy_offscreen(X2, Y2, W, H, pxm->id_, cx + (X2 - X), cy + (Y2 - Y));
+      }
+      XDestroyRegion(r2);
+    }
   }
-  copy_offscreen(X, Y, W, H, pxm->id_, cx, cy);
+  if (!complex_clip) copy_offscreen(X, Y, W, H, pxm->id_, cx, cy);
   if (pxm->mask_) {
     // put the old clip region back
     XSetClipOrigin(fl_display, fl_gc, 0, 0);
-    fl_restore_clip();
+    restore_clip();
   }
 }
 
+
 //------------------------------------------------------------------------------
 #endif							// (platform-specific)
 //------------------------------------------------------------------------------
